@@ -32,9 +32,11 @@
 #ifdef HAVE_XWIN_CONFIG_H
 #include <xwin-config.h>
 #endif
+
 #include "win.h"
 #include <shellapi.h>
 #include "winprefs.h"
+#include "winclipboard/winclipboard.h"
 
 /*
  * Initialize the tray icon
@@ -107,9 +109,7 @@ LRESULT
 winHandleIconMessage(HWND hwnd, UINT message,
                      WPARAM wParam, LPARAM lParam, winPrivScreenPtr pScreenPriv)
 {
-#if defined(XWIN_MULTIWINDOWEXTWM) || defined(XWIN_MULTIWINDOW)
     winScreenInfo *pScreenInfo = pScreenPriv->pScreenInfo;
-#endif
 
     switch (lParam) {
     case WM_LBUTTONUP:
@@ -144,7 +144,6 @@ winHandleIconMessage(HWND hwnd, UINT message,
         /* Get actual tray icon menu */
         hmenuTray = GetSubMenu(hmenuPopup, 0);
 
-#ifdef XWIN_MULTIWINDOW
         /* Check for MultiWindow mode */
         if (pScreenInfo->fMultiWindow) {
             MENUITEMINFO mii = { 0 };
@@ -164,12 +163,24 @@ winHandleIconMessage(HWND hwnd, UINT message,
             SetMenuItemInfo(hmenuTray, ID_APP_HIDE_ROOT, FALSE, &mii);
         }
         else
-#endif
         {
             /* Remove Hide Root Window button */
             RemoveMenu(hmenuTray, ID_APP_HIDE_ROOT, MF_BYCOMMAND);
         }
 
+        if (g_fClipboard) {
+            /* Set menu state to indicate if 'Monitor Primary' is enabled or not */
+            MENUITEMINFO mii = { 0 };
+            mii.cbSize = sizeof(MENUITEMINFO);
+            mii.fMask = MIIM_STATE;
+            mii.fState = fPrimarySelection ? MFS_CHECKED : MFS_UNCHECKED;
+            SetMenuItemInfo(hmenuTray, ID_APP_MONITOR_PRIMARY, FALSE, &mii);
+        }
+        else {
+            /* Remove 'Monitor Primary' menu item */
+            RemoveMenu(hmenuTray, ID_APP_MONITOR_PRIMARY, MF_BYCOMMAND);
+        }
+
         SetupRootMenu(hmenuTray);
 
         /*
