@@ -26,7 +26,7 @@
 
 #include "present_priv.h"
 
-RESTYPE present_event_type;
+static RESTYPE present_event_type;
 
 static int
 present_free_event(void *data, XID id)
@@ -91,6 +91,7 @@ present_event_swap(xGenericEvent *from, xGenericEvent *to)
         swapl(&c->serial);
         swapll(&c->ust);
         swapll(&c->msc);
+        break;
     }
     case PresentIdleNotify:
     {
@@ -99,6 +100,7 @@ present_event_swap(xGenericEvent *from, xGenericEvent *to)
         swapl(&c->window);
         swapl(&c->serial);
         swapl(&c->idle_fence);
+        break;
     }
     }
 }
@@ -208,14 +210,37 @@ present_send_idle_notify(WindowPtr window, CARD32 serial, PixmapPtr pixmap, stru
 int
 present_select_input(ClientPtr client, XID eid, WindowPtr window, CARD32 mask)
 {
-    present_window_priv_ptr window_priv = present_get_window_priv(window, mask != 0);
+    present_window_priv_ptr window_priv;
     present_event_ptr event;
+    int ret;
+
+    /* Check to see if we're modifying an existing event selection */
+    ret = dixLookupResourceByType((void **) &event, eid, present_event_type,
+                                 client, DixWriteAccess);
+    if (ret == Success) {
+        /* Match error for the wrong window; also don't modify some other
+         * client's event selection
+         */
+        if (event->window != window || event->client != client)
+            return BadMatch;
 
-    if (!window_priv) {
         if (mask)
-            return BadAlloc;
+            event->mask = mask;
+        else
+            FreeResource(eid, RT_NONE);
         return Success;
     }
+    if (ret != BadValue)
+        return ret;
+
+    if (mask == 0)
+        return Success;
+
+    LEGAL_NEW_RESOURCE(eid, client);
+
+    window_priv = present_get_window_priv(window, TRUE);
+    if (!window_priv)
+        return BadAlloc;
 
     event = calloc (1, sizeof (present_event_rec));
     if (!event)
